;
; TinyBee-3 test program for MiniDragon Plus2 (16MHz)
; Author: Wayne Chu, (c)2008, Wytec
;   Date: 10/15/2008
;
; Test procedure:
;      1. Connect the 16x2 LCD module to J1
;      2. Connect the TinyBee-3 to J5
;      3. Set the DIP switch S7 for EVB mode ( both switches are down)
;      4. Press reset button on MiniDragon Plus2 and D-Bug12 sign-on message
;         will show on PC screen.
;      5. Enter Load command and download the TinyBee.s19 into EEPROM
;      6. Set the DIP switch S7 for Jump-to-EE mode
;      7. Press reset button, the LCD should display the values of X-Y-Z axes.
;
;         The VCC of the TinyBee-3 module is 3.3V.  In a 3.3V system,
;         1.65V (half of the 3.3V) represents zero acceleration or in
;         horizontal position.  When this accelerometer is tilted,
;         the voltage range is between 1.23V to 2.11V. With 19.53mv per count
;         the count range is 63 to 108.

;
;
MULTI_MODE      equ        $10
SINGLE_MODE     equ        0
SCAN_MODE       equ        $20
NO_SCAN_MODE    equ        0
CHANNEL_NUM_X:  equ        0        ; reading input from AN08
CHANNEL_NUM_Y:  equ        1        ; reading input from AN09
CHANNEL_NUM_Z:  equ        2        ; reading input from AN10
;        
;
;
;  Display a message on the 16X2 LCD display module. The cursor is off.
;
;  This sample program uses 4-bit transfer via port K:
;  PK0 ------- RS ( register select, 0 = register transfer, 1 = data transfer).
;  PK1 ------- Enable ( write pulse )
;  PK2 ------- Data Bit 4 of LCD
;  PK3 ------- Data Bit 5 of LCD
;  PK4 ------- Data Bit 6 of LCD
;  PK5 ------- Data Bit 7 of LCD
;
; Timing of 4-bit data transfer is shown on page 11 of the Seiko LCD
; application note on the distribution CD. The file name is Seikolcd.pdf
;

ONE_MS:         equ        4000        ; 4000 x 250ns = 1 ms at 24 MHz bus speed
FIVE_MS:        equ        20000
TEN_MS:         equ        40000
FIFTY_US:       equ        200

;

DB0:            equ        1
DB1:            equ        2
REG_SEL:        equ        DB0        ; 0=reg, 1=data
NOT_REG_SEL:    equ        $FE
ENABLE:         equ        DB1
NOT_ENABLE:     equ        $FD

LCD:            equ        portk
LCD_RS:         equ        portk
LCD_EN:         equ        portk

REGBLK:         equ        $0

#include        reg9s12.h        ; include register equates

                org        $1000
;

value_x:        rmb        1
value_y:        rmb        1
value_z:        rmb        1
adctl_image:    rmb        1

pkimg:          rmb        1
temp1:          rmb        1

LCDimg:         equ        pkimg
LCD_RSimg:      equ        pkimg
LCD_ENimg:      equ        pkimg

msg2:           rmb        16

STACK:          equ        $2000
;
;
        org        $400
        jmp        start


MSG1:   FCC     "  X    Y    Z   "
;msg2           " 255  255  255  "
;msg2           "0123456789012345"

; this routine will read adc input on AN08, AN09 and AN10 and store the result
; adr10h, adr11h and adr12h
;
;
a_bcd:  clrb
babcd3  clr     $0,x
        clr     $1,x
        clr     $2,x
bd1     suba    #100
        sbcb    #0
        bmi     bd2
        inc     $0,x
        bra     bd1
bd2     adda    #100
        adcb    #0
bd3     suba    #10
        sbcb    #0
        bmi     bd4
        inc     $1,x
        bra     bd3
bd4     adda    #10
        adcb    #0
        staa    $2,x
;
        ldaa    0,x             ; convert to ASCII
        adda    #$30
        staa    0,x
        ldaa    1,x             ; convert to ASCII
        adda    #$30
        staa    1,x
        ldaa    2,x             ; convert to ASCII
        adda    #$30
        staa    2,x
        rts

adc_conv:
        adda        #SINGLE_MODE+NO_SCAN_MODE
;
; if you want to read multi-channel input, change above statement to
;        adda        #MULTI_MODE+NO_SCAN_MODE
;
        staa       adctl_image
        ldx        #REGBLK
        jsr        conv
        rts
;        
conv:   ldaa        adctl_image
        staa        atd1ctl5,x
not_ready:
        brclr       atd1stat,x $80 not_ready
        rts
        
start:  lds         #STACK

;       The internal PLL clock lets us set the speed of the processor.
;;      If you do not set the speed of the PLL as we are doing here, the
;;      default bus speed speed will be 4 Mhz (half the OscFreq).
;;
;;      The math used to set the PLL frequency:
;;        OscFreq = 8Mhz
;;        initSYNR = 2      PLL multiplier
;;        initREFDV = 0     PLL divider - 1
;;        PLLCLK = 2 * OscFreq * (initSYNR+1) / (initREFDV+1)
;;      Usually the OscFreq is 8Mhz, so we have PLLCLK = 2*8*3/1 = 48Mhz
;;      The bus clock runs at half of the PLL speed:
;;      Bus Clock = PLLCLK / 2 = 24 Mhz (for an OscFreq of 8 Mhz)
;;
;;      The effective cycle clock you should use for timing instructions is the
;;      bus clock of 24 Mhz. This is currently the fastest speed this device
;;      can safely use. Overclocking is not recommended.
;;
;;      If you set a slower speed, it will reduce power consumption. If low
;;      power consumption is important (like for battery-powered circuits), you
;;      should also put the processor to sleep as often as possible.
;;
;;      Set PLL speed:
;;
;********PLL code for 24MHz bus speed from a 4/8/16 crystal
        sei
        ldx        #REGBLK
        bclr        clksel,x,%10000000        ; clear bit 7, clock derived from oscclk
        bset        pllctl,x,%01000000        ; Turn PLL on, bit 6 =1 PLL on, bit 6=0 PLL off
        ldaa        #$05                        ; 5+1=6 multiplier
        staa        synr,x
        ldaa        #$03                        ; 3+1=4 divisor, 6/4 * 16 = 24Mhz for 16 MHz crystal
;        ldaa        #$01                        ; 1+1=2 divisor, 6/2 *  8 = 24Mhz for  8 MHz crystal
;        ldaa        #$00                        ; 0+1=1 divisor, 6/1 *  4 = 24Mhz for  4 MHz crystal
        staa        refdv,x
wait_b3:
        brclr        crgflg,x, %00001000 wait_b3         ; Wait until bit 3 = 1
        bset        clksel,x, %10000000

           jsr            delay_10ms                ; delay 20ms during power up
            jsr            delay_10ms

            jsr            lcd_ini                ; initialize the LCD
        ldx         #REGBLK

        bset        atd1ctl2,x $80        ; enable adc operation
        bset        atd1ctl3,x $40        ; 8 conversion needed for an07

        ldx                #MSG1                ; MSG1 for line1, x points to MSG1
        ldab        #16                   ; send out 16 characters
            jsr            lcd_line1

        

begin:  ldaa        #CHANNEL_NUM_X        ; set channel number before calling
        jsr         adc_conv
        ldaa        adr10h+REGBLK
        staa        value_x

        ldaa        #CHANNEL_NUM_Y        ; set channel number before calling
        jsr         adc_conv
        ldaa        adr11h+REGBLK
        staa        value_y
        
        ldaa        #CHANNEL_NUM_Z        ; set channel number before calling
        jsr         adc_conv
        ldaa        adr12h+REGBLK
        staa        value_z

            ldx            #msg2                        ; MSG2 for line2, x points to MSG2
clear_next:
        ldaa        #$20                        ; fill with space
        staa        0,x
        inx
        cpx     #msg2+16
        bne     clear_next
        ldaa    value_x
        ldx     #msg2+1                 ; starting location for X-axis
        jsr     a_bcd                   ; convert accu A to 3 digit BCD
        
         ldaa    value_y
        ldx     #msg2+6                 ; starting location for Y-axis
        jsr     a_bcd                   ; convert accu A to 3 digit BCD
        
         ldaa    value_z
        ldx     #msg2+11                ; starting location for Z-axis
        jsr     a_bcd                   ; convert accu A to 3 digit BCD
        
        ldx     #msg2
        ldab    #16                     ; send out 16 characters
             jsr        lcd_line2
        ldaa    #25
delay_more:
        jsr        delay_10ms
        deca
        bne     delay_more
        
               jmp        begin

lcd_ini:
        ldaa        #$ff
        staa        ddrk                ; port K = output
        clra
        staa        pkimg
        staa        portk

               ldx        #inidsp         ; point to init. codes.
        pshb                    ; output instruction command.
               jsr            sel_inst
               ldab           0,x
               inx
onext:        ldaa           0,x
               jsr            wrt_nibble         ; initiate write pulse.
               inx
        jsr        delay_5ms       ; every nibble is delayed for 5ms
               decb                    ; in reset sequence to simplify coding
               bne            onext
               pulb
               rts

inidsp: fcb        12                ; number of high nibbles
*                                ; use high nibbles only, low nibbles are ignored
        fcb        $30                ; 1st reset code, must delay 4.1ms after sending
        fcb        $30                ; 2nd reste code, must delay 100us after sending

; all following 10 nibbles must be delay 40us each after sending
        fcb     $30             ; 3rd reset code,
        fcb        $20                ; 4th reste code,
        fcb        $20                   ; 4 bit mode, 2 line, 5X7 dot
        fcb        $80                   ; 4 bit mode, 2 line, 5X7 dot
        fcb        $00                ; cursor increment, disable display shift
        fcb        $60                ; cursor increment, disable display shift
        fcb        $00                ; display on, cursor off, no blinking
        fcb        $C0                ; display on, cursor off, no blinking
        fcb        $00                ; clear display memory, set cursor to home pos
        fcb        $10                ; clear display memory, set cursor to home pos
*
sel_data:
        psha
;        bset        LCD_RSimg REG_SEL        ; select instruction
        ldaa        LCD_RSimg
        oraa        #REG_SEL
        bra        sel_ins
sel_inst:
        psha
;        bclr        LCD_RSimg REG_SEL        ; select instruction
        ldaa        LCD_RSimg
        anda        #NOT_REG_SEL
sel_ins:
        staa        LCD_RSimg
        staa        LCD_RS
        pula
        rts

lcd_line1:
        jsr            sel_inst                ; select instruction
               ldaa           #$80                     ; starting address for the line1
        bra        line3
lcd_line2:
        jsr            sel_inst
               ldaa           #$C0                     ; starting address for the line2
line3:         jsr            wrt_byte

               jsr            sel_data
        jsr        msg_out
        rts
;
; at entry, x must point to the begining of the message,
;           b = number of the character to be sent out

msg_out:
        ldaa           0,x
               jsr            wrt_byte
               inx
               decb
               bne            msg_out
        rts

wrt_nibble:
               anda           #$f0                     ; mask out 4 low bits
               lsra
        lsra                            ; 4 MSB bits go to pk2-pk5
        staa           temp1                         ; save high nibble
               ldaa           LCDimg                    ; get LCD port image
               anda           #$03                     ; need low 2 bits
               oraa           temp1                    ; add it with high nibble
        staa           LCDimg                    ; save it
        staa           LCD                            ; output data to LCD port
        jsr        enable_pulse
         rts
*

;       @ enter, a=data to output
;
wrt_byte:
        pshx
               psha                                ; save it tomporarily.
               anda   #$f0                     ; mask out 4 low bits.
        lsra
        lsra                                ; 4 MSB bits go to pk2-pk5
               staa   temp1                         ; save nibble value.
               ldaa   LCDimg                    ; get LCD port image.
               anda   #$03                     ; need low 2 bits.
               oraa   temp1                    ; add in low 4 bits.
               staa   LCDimg                    ; save it
               staa   LCD                            ; output data
;
        bsr        enable_pulse
        pula
         asla                            ; move low bits over.
        asla
               staa           temp1                        ; store temporarily.
;
               ldaa           LCDimg                         ; get LCD port image.
               anda           #$03                           ; need low 2 bits.
               oraa           temp1                          ; add in loiw 4 bits.
               staa           LCDimg                    ; save it
               staa           LCD                            ; output data
;
        bsr        enable_pulse
        jsr        delay_50us
         pulx
               rts
;
enable_pulse:
;        bset        LCD_ENimg ENABLE        ; ENABLE=high
        ldaa        LCD_ENimg
        oraa        #ENABLE
        staa        LCD_ENimg
        staa        LCD_EN

;        bclr        LCD_ENimg ENABLE        ; ENABLE=low
        ldaa        LCD_ENimg
        anda        #NOT_ENABLE
        staa        LCD_ENimg
        staa        LCD_EN
        rts

delay_10ms:
        pshx
        ldx     #TEN_MS
          bsr        del1
        pulx
        rts
delay_5ms:
        pshx
        ldx     #FIVE_MS
          bsr        del1
        pulx
        rts
delay_50us:
        pshx
        ldx     #FIFTY_US
          bsr        del1
        pulx
        rts
;
; 250ns delay at 24MHz bus speed
;
del1:        dex                                ; 1 cycle
        inx                                ; 1 cycle
        dex                                ; 1 cycle
        bne        del1                        ; 3 cylce
        rts

        end
